@@ -1,3 +1,4 @@
+from math import floor
 from django.forms import BaseForm
 from django.forms.forms import BoundField
 from django.forms.widgets import TextInput, CheckboxInput, CheckboxSelectMultiple, RadioSelect
@@ -143,21 +144,56 @@ def active_url(request, url, output=u'active'):
     return ''
 
 @register.filter
-def pagination(page, range=5):
+def pagination(page, pages_to_show=11):
     """
     Generate Bootstrap pagination links from a page object
     """
+    pages_to_show = int(pages_to_show)
+    if pages_to_show < 1:
+        raise ValueError("Pagination pages_to_show should be a positive integer, you specified %s" % pages_to_show)
     num_pages = page.paginator.num_pages
     current_page = page.number
-    range_min = max(current_page - range, 1)
-    range_max = min(current_page + range, num_pages)
+    half_page_num = int(floor(pages_to_show / 2)) - 1
+    if half_page_num < 0:
+        half_page_num = 0
+    first_page = current_page - half_page_num
+    if first_page <= 1:
+        first_page = 1
+    if first_page > 1:
+        pages_back = first_page - half_page_num
+        if pages_back < 1:
+            pages_back = 1
+    else:
+        pages_back = None
+    last_page = first_page + pages_to_show - 1
+    if pages_back is None:
+        last_page += 1
+    if last_page > num_pages:
+        last_page = num_pages
+    if last_page < num_pages:
+        pages_forward = last_page + half_page_num
+        if pages_forward > num_pages:
+            pages_forward = num_pages
+    else:
+        pages_forward = None
+        if first_page > 1:
+            first_page -= 1
+        if pages_back > 1:
+            pages_back -= 1
+        else:
+            pages_back = None
+    pages_shown = []
+    for i in range(first_page, last_page + 1):
+        pages_shown.append(i)
     return get_template("bootstrap_toolkit/pagination.html").render(
         Context({
-            'page': page,
             'num_pages': num_pages,
             'current_page': current_page,
-            'range_min': range_min,
-            'range_max': range_max,
+            'first_page': first_page,
+            'last_page': last_page,
+            'pages_shown': pages_shown,
+            'pages_back': pages_back,
+            'pages_forward': pages_forward,
         })
     )
 
